using System;
using System.Collections.Generic;
using System.Net.Mime;
using System.Security.Cryptography;
using FarseerPhysics.Dynamics.Joints;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using Silk.NET.SDL;
using Color = Jypeli.Color;

namespace KelaCombatTesti;

/// @author Sakari
/// @version 17.02.2025
/// <summary>
/// 
/// </summary>
    public class KelaCombatTesti : PhysicsGame
    {
        PlatformCharacter pelaaja;
        PhysicsObject Pesismaila;
        PhysicsObject Viinapullo;
        PlatformCharacter Kelabotti;
        private Image TiiliseinaKuva = LoadImage("tiiliseina");
        Image KelabottiKuva = LoadImage("Kelabotti");
        Image PelaajaKuva = LoadImage("Pelaaja.png");
        List<PhysicsObject> spawnipaikat = new List<PhysicsObject>(); // tässä listataan mahdolliset paikat, johon kelabotti voi spawnaa. Lista pitää sisällään spawnereita.
        
        //IntMeter edustaa muuttujia, jotka voivat kasvaa tai vähentyä tietyllä tavalla 
        IntMeter pisteet;  //pelaajan keräämät pisteet 
        DoubleMeter pelaajanelamaLaskuri; //pelaajan elämät
        IntMeter Kelabotinelamalaskuri; //kelabotin elämät
        
        //Label = tekstielementti, joka näytetään pelissä 
        Label pisteNaytto; //pisteiden näyttäminen ruudulla 

        
        
      
        /// <summary>
        /// 
        /// </summary>
        public override void Begin()
        {
            LuoKentta();
            LuoElamalaskuri();
            LuoPistelaskuri(); // Alustetaan pistelaskuri 
            
            
            //Ajastin, joka lisää uuden vihollisen joka 5. sekunti 
           
            
            //kelabotin elämien väheneminen VahingoitaKelabottia aliohjelman mukaisesti
            //AddCollisionHandler(Kelabotti, "Pesismaila", VahingoitaKelabottia);

        
            Camera.Follow(pelaaja);
            
            Keyboard.Listen(Key.A, ButtonState.Down, LiikuVasemmalle, null, new Vector());
            Keyboard.Listen(Key.D, ButtonState.Down, LiikuOikealle, null, new Vector());
            Keyboard.Listen(Key.W, ButtonState.Down, Hyppaa, null, new Vector());
            //Keyboard.Listen(Key.S, ButtonState.Down, LiikutaPelaajaa, null, new Vector(0, -1000));
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
            
            Keyboard.Listen(Key.Space, ButtonState.Pressed, LuoPesismaila, "", new Vector(90, 10), pelaaja);
           

            
        
            void LiikuVasemmalle(Vector vektori)
            {
                pelaaja.Walk(-120);
            }

            void LiikuOikealle(Vector vektori)
            {
                pelaaja.Walk(120);
            }
            
            void Hyppaa(Vector vektori)
            {
                pelaaja.Jump(280.0);
            }
        }
        
        
        /// <summary>
        /// Tässä luodaan pelikenttä
        /// </summary>
        void LuoKentta()
        {

            Level.Background.CreateGradient(Color.DarkGray, Color.Blue);
            Gravity = new Vector(0, -1100.0);
            
           
            // pelissä kentät luodaan piirtämällä pikseleitä. Pikselin väri kuvaa aina jotain tiettyä asiaa kentällä
            // ruutu = pikseli
            ColorTileMap ruudut = ColorTileMap.FromLevelAsset("kentta1");
            
            ruudut.SetTileMethod(Color.Lime, LuoPelaaja); //LIME pikseli on pelaaja
            ruudut.SetTileMethod(Color.Black, LuoTaso); //MUSTA on tiiliseinää
            ruudut.SetTileMethod(Color.Red, LuoKelabottiSpawni); //PUNAISEEN voi spawnaa vihu
            ruudut.SetTileMethod(Color.SkyBlue, LuoViinapullo); //VAALEANSNISET on pelaajaa parantavia viinapulloja
            
            ruudut.Execute(80, 80);

            
            
            
            Timer.CreateAndStart(5.0, LuoKelabotti);
        }
        //tämä aliohjelma luo tiili-palikoita, joista kenttä (tällä hetkellä) rakentuu
        void LuoTaso(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
            taso.Position = paikka;
            taso.CollisionIgnoreGroup = 2;
            taso.Color = Color.Gray;
            taso.Image = TiiliseinaKuva;
            Add(taso);
        }
        //tämä aliohjelma luo pelaajaa parantavan viinapllon
        void LuoViinapullo(Vector paikka, double x, double y)
        {
            Viinapullo = PhysicsObject.CreateStaticObject(20, 20);
            Viinapullo.Position = paikka;
            Viinapullo.Color = Color.White;
            Viinapullo.Tag = "Viinapullo";
            Add(Viinapullo);

        }

        //tämä aliohjelma luo "haamupalikan" jonka sijaintiin kelabotti myöhemmin luodaan
        void LuoKelabottiSpawni(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject BottiSpawni = PhysicsObject.CreateStaticObject(leveys, korkeus);
            BottiSpawni.Position = paikka;
            BottiSpawni.CollisionIgnoreGroup = 1;
            BottiSpawni.IsVisible = false;
            BottiSpawni.IgnoresGravity = true;
            Add(BottiSpawni, -1);
            spawnipaikat.Add(BottiSpawni);
        }
        
      
        /// <summary>
        /// Aliohjelma, jossa luodaan pelaaja
        /// </summary>
        /// <param name="x">pelaajan x-koordinaatti</param>
        /// <param name="y">pelaajan y-koordinaatti</param>
        /// <returns>pelaaja</returns>
        void LuoPelaaja(Vector paikka, double x, double y)
        {
            pelaaja = new PlatformCharacter(50.0, 100.0);
            pelaaja.Image = PelaajaKuva;
            pelaaja.Mass = 1.0;
            pelaaja.Shape = Shape.Rectangle;
            pelaaja.Color = Color.Black;
            pelaaja.Position = paikka;
            pelaaja.CollisionIgnoreGroup = 1;
          
            // Törmäyksen käsittelijä, mitä tapahtuu, kun pelaaja osuu kelabotiin 
            //Siis pelaajan elämien väheneminen VahnigoitaPelaajaa-aliohjelman mukaisesti 
            AddCollisionHandler(pelaaja, "Kelabotti", VahingoitaPelaajaa);
            AddCollisionHandler(pelaaja, "Viinapullo", ParannaPelaajaa);
            AddCollisionHandler(pelaaja, "kappale", VahingoitaPelaajaa);
            
            Add(pelaaja);
            
        }
        
        /// <summary>
        /// Aliohjelma, joka luo pelaajalle elämälaskurin ja sen nimen: "Mielenterveys"
        /// </summary>
        void LuoElamalaskuri()
        {
            pelaajanelamaLaskuri = new DoubleMeter(100.0);
            pelaajanelamaLaskuri.MaxValue = 100;
            pelaajanelamaLaskuri.LowerLimit += ElamaLoppui;

            ProgressBar elamapalkki = new ProgressBar(150, 18);
            elamapalkki.X = Screen.Left + 110;
            elamapalkki.Y = Screen.Bottom + 50;
            elamapalkki.BindTo(pelaajanelamaLaskuri);
            elamapalkki.Color = Color.Red;
            elamapalkki.BarColor = Color.Green;
            elamapalkki.BorderColor = Color.Black;
            Label tekstikentta = new Label(200, 10, "Mielenterveys");
            tekstikentta.X = Screen.Left + 100;
            tekstikentta.Y = Screen.Bottom + 70;
            tekstikentta.Font = new Font(20);
            Add(tekstikentta);
            Add(elamapalkki);

        }
        

       
        /// <summary>
        /// Aliohjelma, joka käsittelee pelaajan kuoleman ja ilmoittaa pelin loppumisen
        /// </summary>
        void ElamaLoppui()
        {
            Label Kuolema = new Label(800, 60, "Toimeentulohakemus hylätty.");
            Kuolema.Y = 140;
            Kuolema.Font = new Font(60);
            Kuolema.TextColor = Color.Red;
            //Kuolema.Color = Color.Black;
            Add(Kuolema);
        }
        

        /// <summary>
        /// Tässä luodaan uusi fysiikkaolio, Pesismaila
        //Tämä toimii pelaajan lyöntinä (samalla määritellään ase.
        /// </summary>
        /// <param name="koko">Pesismailan koko</param>
        /// <param name="pelaaja">kohde, johon Pesismaila liitetään, eli pelaaja</param>
        void LuoPesismaila(Vector koko, PlatformCharacter pelaaja)
        {
            PhysicsObject Pesismaila = new PhysicsObject(koko.X, koko.Y);
            Pesismaila.Position = pelaaja.Position + new Vector(80*pelaaja.FacingDirection.GetVector().X, 10);
            Pesismaila.CollisionIgnoreGroup = 1;
            Pesismaila.Mass = 1;
            Pesismaila.Tag = "Pesismaila";
            Add(Pesismaila);
            Pesismaila.LifetimeLeft = TimeSpan.FromSeconds(0.075);
            Pesismaila.IgnoresGravity = true;
            
            //PISTEET ei elämät
            AddCollisionHandler(Pesismaila, "Kelabotti", KunVihollinenTuhoutuu);
            
        }

        
        /// <summary>
        /// Tämä funktio luo Kelabotti-vihun ja sille annetaan pelaajan seuraamis aivot
        /// </summary>
        /// <returns>Kelabotti</returns>
        void LuoKelabotti()
        {
            PhysicsObject spawni = spawnipaikat[RandomGen.NextInt(spawnipaikat.Count)];
            PlatformCharacter Kelabotti = new PlatformCharacter(80.0, 80.0);
            Kelabotti.Color = Color.Orange;
            Kelabotti.Image = KelabottiKuva;
            Kelabotti.CollisionIgnoreGroup = 3;
            Kelabotti.MirrorImage();
            Kelabotinelamalaskuri = new IntMeter(5); // Alussa on 5 elämää 
            Kelabotti.Tag = "Kelabotti";
            Kelabotti.Position = spawni.Position;
            Kelabotti.Weapon = new AssaultRifle(30, 10);
            Kelabotti.Weapon.ProjectileCollision = VahingoitaPelaajaa;
            Add(Kelabotti);
            
            
            FollowerBrain seuraajanAivot = new FollowerBrain(pelaaja);
            seuraajanAivot.Speed = 100;

            Kelabotti.Brain = seuraajanAivot;
            
            Timer heittoajastin = new Timer();
            heittoajastin.Interval = 2;
            heittoajastin.Timeout += delegate() { KelabotinAmmus(Kelabotti, "pelaaja"); };
            heittoajastin.Start();

            // Destroyed-tapahtuma Jypelissä:
            // https://tim.jyu.fi/view/kurssit/jypeli/tapahtumat/muuttapahtumat
            Kelabotti.Destroyed += heittoajastin.Stop;

            
        }
       /// <summary>
       /// Aliohjelma, jossa käsitellään Kelabotin heittämä ammus
       /// </summary>
       /// <param name="heittavaOlio">Kelabotti heittää ammuksen</param>
       /// <param name="kohdeolionTunniste">pelaajan tunniste</param>
        public void KelabotinAmmus(PhysicsObject heittavaOlio, string kohdeolionTunniste)
        {
            PhysicsObject kappale = new PhysicsObject(15, 15, Shape.Circle);
            //kappale.Hit(new Vector(180,80));
            kappale.Hit(new Vector());
            kappale.Position = heittavaOlio.Position;
            kappale.CollisionIgnoreGroup = 3;
            kappale.Color = Color.Silver;
            kappale.Tag = "kappale";
            kappale.MaximumLifetime = TimeSpan.FromSeconds(8);
            Add(kappale);
        }

        
        /// <summary>
        /// Tämä aliohjelma käsittlee sen, kun pelaaja osuu Kelabottiin
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="Kelabotti">pelin vihu; Kelabotti</param>
        void VahingoitaPelaajaa(PhysicsObject pelaaja, PhysicsObject kappale)
        {
            int luku = RandomGen.NextInt(19, 34);
            pelaajanelamaLaskuri.Value = pelaajanelamaLaskuri - luku; // Vähentää pelaajan elämää satunnaisen määrän
            //MessageDisplay.Add("Osuma! Elämät jäljellä: " + pelaajanelamaLaskuri.Value); 
            
            if (pelaajanelamaLaskuri.Value <= 0) //jos laskuri menee nollaan... 
            { 
                pelaaja.Destroy(); //...ja pelaaja tuhotaan
                //Pesismaila.Destroy();//...myös pesismaila
            } 
            
        }
        
        //tämä aliohjelma määrää, mitä käy, kun pelaaja koskee viinapulloa

        void ParannaPelaajaa(PhysicsObject pelaaja, PhysicsObject Viinapullo)
        {
            //Viinapullo.CollisionIgnoreGroup = 1;
            
            if (pelaajanelamaLaskuri.Value < 90)
            {
                //Viinapullo.CollisionIgnoreGroup = 0;
                pelaajanelamaLaskuri.Value = pelaajanelamaLaskuri.Value + 20;
                Viinapullo.Destroy();
            }
            
            
        }

        //tää ei toimi vielä
        void VahingoitaKelabottia(PhysicsObject Kelabotti, PhysicsObject Pesismaila) 
        { 
            Kelabotinelamalaskuri.Value--; // Vähentää kelabotin elämää yhdellä 
            if (Kelabotinelamalaskuri.Value <= 0) //jos laskuri menee nollaan... 
            { 
                Kelabotti.Destroy(); //...ja kelabotti tuhotaan
            } 
        } 
        /// <summary>
        /// Luo pistelaskurin pisteille, joita saa kun hakkaa kelabotteja
        /// </summary>
        void LuoPistelaskuri() 
        { 
            
            Label opintoNopat = new Label(150, 50, "Opintonopat:");
            opintoNopat.X = Screen.Left + 100;
            opintoNopat.Y = Screen.Bottom + 93;
            opintoNopat.Font = new Font(20);
            opintoNopat.TextColor = Color.Black;
            Add(opintoNopat);
            
            pisteet = new IntMeter(0); // Aloitetaan nollasta 
            pisteNaytto = new Label(); 
            pisteNaytto.X = Screen.Left + 170;  
            pisteNaytto.Y = Screen.Bottom + 95;  
            pisteNaytto.TextColor = Color.OrangeRed; 
            pisteNaytto.Font = new Font(20); 
            pisteNaytto.BindTo(pisteet); // Päivitetään teksti automaattisesti 
            Add(pisteNaytto);
            
            
        } 
        
        /// <summary>
        /// Käsittelee kelabotin tuhoamisen ja antaa siitö pisteitä
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="kelabotti">pelin vihu; Kelabotti</param>
        void KunVihollinenTuhoutuu(PhysicsObject pelaaja, PhysicsObject kelabotti) 
        { 
            pisteet.Value += 100; // Lisää esim. 100 pistettä, kun botti tuhotaan
            kelabotti.Destroy(); // Poistaa botin näytöltä
        } 
        
    }

        
    


